function [J,A,r,rIndiv] = JIVE_RankSelect(Datatypes,alpha,Nperm,scale, convThresh,MaxIter)
%[J,A,r,rIndiv] = JIVE_RankSelect(Datatypes,alpha,Nperm,scale, convThresh)
%Input:
%    - Datatypes: Cell array of data matrices with matching columns (Datatypes{i} gives the i'th data matrix)
%   - alpha: Significance threshold
%   - Nperm: Number of permutations
%   - scale: Should the datasets be centered and scaled by total variation? (default = 'y')
%   - convThresh:  Convergence threshold to use for JIVE algorithm (default = 10^(-8))
%   - MaxIter: Maximum numeber of iterations (default = 1000)
%Output:
%   -J,A: JIVE estimates
%   - r: Rank of joint structure
%   - rIndiv: Vector of ranks for individual structure.


if(class(Datatypes)~='cell')
    disp('Error: Datatypes should be of class ''cell''');
    disp('For data matrices X1,X2,...,Xk, input Datatypes = {X1,X2,...,Xk}');
end
nDatatypes = length(Datatypes);
for i=1:nDatatypes
    [d(i),n(i)] =size(Datatypes{i});
end
n = unique(n);
if(length(n) ~= 1)
    disp('Error: Datatypes do not have same number of columns');
end

if(nargin<4)
    scale = 'y';
end

if(scale =='y')
    for(i=1:nDatatypes)
        Datatypes{i} = bsxfun(@minus,Datatypes{i},mean(Datatypes{i}')');
        Datatypes{i} = Datatypes{i}/norm(Datatypes{i}, 'fro');
    end
end

if(nargin<5)
   convThresh = 10^(-8);
end

if(nargin<6)
   MaxIter = 1000;
end
       

%Estimate ranks
A =zeros(sum(d),n);
J=zeros(sum(d),n);
rPrev=1;
rIndivPrev=ones(1,nDatatypes);
r=0;
rIndiv = zeros(1,nDatatypes);
First = true;
Resid = Datatypes;
SingValsI = cell(1,nDatatypes);
SingValsPermI = cell(1,nDatatypes);
PermThreshI =cell(1,nDatatypes); 
ResidPerm = cell(1,nDatatypes);

while (r ~= rPrev || ~all(rIndivPrev==rIndiv))
    rPrev=r;
    rIndivPrev=rIndiv;
[U,S,V] = svds(J,rPrev);
for(k=1:nDatatypes)
    Resid{k} = Datatypes{k}-J((sum(d(1:(k-1)))+1):sum(d(1:k)),:);
    SingValsI{k} = svd(Resid{k} - Resid{k}*V*V');
    SingValsPermI{k} = zeros(Nperm,length(SingValsI{k}));
    for(i=1:Nperm)
        ResidPerm{k} = Resid{k};
        for(j=1:d(k))
            ResidPerm{k}(j,:) = ResidPerm{k}(j,randperm(n));
        end
        SingValsPermI{k}(i,:) = svd(ResidPerm{k}-ResidPerm{k}*V*V');
    end
    PermThreshI{k} = prctile(SingValsPermI{k},(1-alpha)*100);
    rIndiv(k)=0;
    for(i=1:min(d(k),n))
        if(SingValsI{k}(i)>PermThreshI{k}(i))   rIndiv(k)=i;
        else break
        end
    end
end

SingVals = svd(cell2mat(Datatypes')-A);
SingValsPerm = zeros(Nperm,min(sum(d),n));
for(i=1:Nperm)
    for(k=1:nDatatypes) 
        Resid{k} = Datatypes{k}-A((sum(d(1:(k-1)))+1):sum(d(1:k)),:);
        ResidPerm{k} = Resid{k}(:,randperm(n));
    end
    SingValsPerm(i,:) = svd(cell2mat(ResidPerm'));
end
PermThresh = prctile(SingValsPerm,(1-alpha)*100);
r=0;
for(i=1:min(sum(d),n))
    if(SingVals(i)>PermThresh(i))   r=i;
    else break
    end
end
r=max(r,rPrev) %%only increase rank of joint structure (generally this results in more accurate rank estimation).  
rIndiv
[J,A] = JIVE(Datatypes,r, rIndiv,scale,convThresh,MaxIter);
end



return;
